@@ -1,14 +1,17 @@
 use strict;
 use warnings;
-package Dist::Zilla::Plugin::DynamicPrereqs; # git description: v0.022-2-gb038cf2
+package Dist::Zilla::Plugin::DynamicPrereqs; # git description: v0.027-4-gf850eba
 # vim: set ts=8 sts=4 sw=4 tw=115 et :
 # ABSTRACT: Specify dynamic (user-side) prerequisites for your distribution
 # KEYWORDS: plugin distribution metadata MYMETA prerequisites Makefile.PL dynamic
 
-our $VERSION = '0.023';
+our $VERSION = '0.028';
 
 use Moose;
 with
+    'Dist::Zilla::Role::FileGatherer',
+    'Dist::Zilla::Role::ModuleIncluder',
+    'Dist::Zilla::Role::ModuleMetadata',
     'Dist::Zilla::Role::FileMunger',
     'Dist::Zilla::Role::MetaProvider',
     'Dist::Zilla::Role::PrereqSource',
@@ -228,11 +231,8 @@ sub _build__sub_definitions
     return $content;
 }
 
+
 my %sub_prereqs = (
-    can_xs => {
-        'ExtUtils::CBuilder' => '0.27',
-        'File::Temp' => '0',
-    },
     can_cc => {
         'Config' => '0',
     },
@@ -249,19 +249,73 @@ my %sub_prereqs = (
     },
 );
 
+# instead of including these dependencies in configure-requires, we inline
+# them right into the distribution in inc/.
+my %sub_inc_dependencies = (
+    can_xs => {
+        'ExtUtils::HasCompiler' => '0.014',
+    },
+);
+
+sub gather_files {
+    my $self = shift;
+
+    foreach my $required_sub ($self->_all_required_subs)
+    {
+        # FIXME: update %included_subs earlier to account for other coexisting
+        # plugins, or running two instances of the plugin will try to do this twice.
+        my $include_modules = $sub_inc_dependencies{$required_sub} || {};
+        foreach my $module (keys %$include_modules)
+        {
+            (my $path = $module) =~ s{::}{/}g;
+            $path = path('inc', $path . '.pm');
+            my $cpath = $path->canonpath;
+
+            my $file = (first { $_->name eq $path or $_->name eq $cpath } @{ $self->zilla->files })
+                # TODO this requires a new release of ModuleIncluder
+                # || ($self->include_modules([ $module ], version->new('5.006001')))[0];
+                || do {
+                    $self->log([ 'inlining %s into inc/', $module ]);
+                    $self->include_modules([ $module ], version->new('5.006001'));
+                    first { $_->name eq $path or $_->name eq $cpath } @{ $self->zilla->files };
+                };
+            warn "failed to find $path in files" if not $file;
+
+            if (defined $include_modules->{$module} and $include_modules->{$module} > 0)
+            {
+                # check that the file we got actually satisfies our dependency
+                my $mmd = $self->module_metadata_for_file($file);
+                $self->log_fatal([ '%s version %s required--only found version %s',
+                        $module, $include_modules->{$module},
+                        (defined $mmd->version ? $mmd->version->stringify : 'undef') ])
+                    if ($mmd->version || 0) < $include_modules->{$module};
+            }
+        }
+    }
+}
+
 sub register_prereqs
 {
     my $self = shift;
     foreach my $required_sub ($self->_all_required_subs)
     {
-        my $prereqs = $sub_prereqs{$required_sub};
+        my $configure_prereqs = $sub_prereqs{$required_sub} || {};
         $self->zilla->register_prereqs(
             {
                 phase => 'configure',
                 type  => 'requires',
             },
-            %$prereqs,
-        ) if $prereqs and %$prereqs;
+            %$configure_prereqs,
+        ) if %$configure_prereqs;
+
+        my $develop_prereqs = $sub_inc_dependencies{$required_sub} || {};
+        $self->zilla->register_prereqs(
+            {
+                phase => 'develop',
+                type  => 'requires',
+            },
+            %$develop_prereqs,
+        ) if %$develop_prereqs;
     }
 }
 
@@ -276,13 +330,13 @@ has _include_sub_root => (
 
 # indicates subs that require other subs to be included
 my %sub_dependencies = (
-    can_xs => [ qw(can_cc) ],
     can_cc => [ qw(can_run) ],
     can_run => [ qw(maybe_command) ],
     requires => [ qw(runtime_requires) ],
     runtime_requires => [ qw(_add_prereq) ],
     build_requires => [ qw(_add_prereq) ],
     test_requires => [ qw(_add_prereq) ],
+    want_pp => [ qw(parse_args) ],
 );
 
 has _all_required_subs => (
@@ -332,7 +386,7 @@ sub _warn_include_subs
 {
     my ($self, @include_subs) = @_;
 
-    $self->log(colored('The use of ' . $_ . ' is not advised. Please consult the documentation!', 'bright_yellow'))
+    $self->log(colored('Use ' . $_ . ' with great care! Please consult the documentation!', 'bright_yellow'))
         foreach grep { exists $warn_include_sub{$_} } @include_subs;
 }
 
@@ -350,7 +404,7 @@ Dist::Zilla::Plugin::DynamicPrereqs - Specify dynamic (user-side) prerequisites
 
 =head1 VERSION
 
-version 0.023
+version 0.028
 
 =head1 SYNOPSIS
 
@@ -358,7 +412,7 @@ In your F<dist.ini>:
 
     [DynamicPrereqs]
     -condition = has_module('Role::Tiny')
-    -condition = !parse_args()->{PUREPERL_ONLY}
+    -condition = !want_pp()
     -condition = can_xs()
     -body = requires('Role::Tiny', '1.003000')
 
@@ -428,7 +482,7 @@ C<[Prereqs / ConfigureRequires]> in your F<dist.ini>).
 
 C<-body> first became available in version 0.018.
 
-=for Pod::Coverage mvp_multivalue_args mvp_aliases BUILD metadata after_build munge_files register_prereqs
+=for Pod::Coverage mvp_multivalue_args mvp_aliases BUILD metadata after_build munge_files register_prereqs gather_files
 
 =head2 C<-delimiter>
 
@@ -511,7 +565,7 @@ C<parse_args()> - returns the hashref of options that were passed as arguments t
 
 =item *
 
-C<can_xs()> - Secondary compile testing via L<ExtUtils::CBuilder>
+C<can_xs()> - XS capability testing via L<ExtUtils::HasCompiler> (don't forget to also check C<want_pp>!)
 
 =item *
 
@@ -569,6 +623,10 @@ C<build_requires($module [, $version ])> - adds the module to runtime prereqs (a
 
 C<test_requires($module [, $version ])> - adds the module to runtime prereqs (as a shorthand for editing the hashes in F<Makefile.PL> directly). Added in 0.016.
 
+=item *
+
+C<want_pp> - true if the user or CPAN client explicitly specified PUREPERL_ONLY (indicating that no XS-requiring modules or code should be installed)
+
 =back
 
 =head1 WARNING: INCOMPLETE SUBROUTINE IMPLEMENTATIONS!
